function [Tracks] = goTracking(fname,minTrackDuration,minTrackLength,pixelsize,useSmoothing,frameinterval,maxspeed,segThreshold, MaxLinkingDistance, MaxGapClosing, minsize,maxsize)
% segments and tracks a ilastik prob. file.
finfo=h5info(fname);

minTrackLength=minTrackLength/pixelsize;  % we work in pixels here ;-)
datasetname=finfo.Datasets.Name;
tips=permute(squeeze(h5read(fname,['/' datasetname])),[1,3,2,4]);
a=max((squeeze(tips(1,:,:,:))),[],3);
b=max((squeeze(tips(2,:,:,:))),[],3);

% identify the tip channel. assumption is that the tip area is smaller than
% the background area
if sum(a(:))>sum(b(:)) 
   tipChannel=2;
else
    tipChannel=1;
end

if isfloat(tips)
    maxProb=1.0;
else
    maxProb=255;
end
tips=(squeeze(double(tips(tipChannel,:,:,:)))/maxProb)>segThreshold;

for i=1:size(tips,3)
    frameProps=regionprops(squeeze(tips(:,:,i)),'Centroid','Area');
    n=0;
    for j=1:length(frameProps)
        if  frameProps(j).Area>=minsize && frameProps(j).Area<=maxsize % discard detections that don't satisfy the size chriteria
            n=n+1;
            points{i}(n,1:2)= frameProps(j).Centroid;
        end
    end
end

[tracks,~,~] = simpletracker(points,'Debug',false,'MaxGapClosing',MaxGapClosing,'MaxLinkingDistance',MaxLinkingDistance);
n=0;
for i=1:length(tracks)
    x=[];y=[];
    if sum(~isnan(tracks{i}(:)))>minTrackDuration % discard too short tracks 
        t=tracks{i}(:);
        for k=1:length(t)
            if isfinite(t(k))
                x(k)=points{k}(t(k),1);
                y(k)=points{k}(t(k),2);
            else
                x(k)=nan;y(k)=nan;
            end
        end
        a=find(~isnan(t));
        x(a(1):a(end))=fillmissing(x(a(1):a(end)),'linear');   % I linearly interpolate missing detections
        y(a(1):a(end))=fillmissing(y(a(1):a(end)),'linear');
        if sqrt((x(a(end))-x(a(1)))^2+(y(a(end))-y(a(1)))^2)>minTrackLength % discard tracks that don't really move
           if useSmoothing
               x=smooth(x,'lowess');  % change the span of the smoothing from the default of 5 here  
               y=smooth(y,'lowess');
           else
               x=x';
               y=y';
           end
           v=nan(size(x));
           alpha=v;
           for k=a(1)+1:a(end)
               v(k)=(sqrt((x(k)-x(k-1))^2+(y(k)-y(k-1))^2)/frameinterval).*pixelsize; % speed in micrometer per minute
               alpha(k)=atan2d(y(k)-y(k-1),x(k)-x(k-1));  
           end    
           if all(v(a(1)+1:a(end))<=maxspeed) % discard tracks that contain unrealistic jumps
               n=n+1;
               Tracks{n}=[x(a(1):a(end)),y(a(1):a(end)),(a(1):a(end))',v(a(1):a(end)),alpha(a(1):a(end))];
           end
        end
    end
end

% plot to test 
% figure;
% hold('on');
% for i=1:length(Tracks)
%     plot(Tracks{i}(:,1),Tracks{i}(:,2));
% end
%     